@@ -26,7 +26,7 @@ INSIDE_DOCKER_BUILD_DIR=/docker_build_dir
 #
 # If these nothing should be passed, then clear or
 #'unset INSIDE_DOCKER_TRAVIS_CI_ENV' after this script is included
-INSIDE_DOCKER_TRAVIS_CI_ENV="TRAVIS TRAVIS_COMMIT TRAVIS_PULL_REQUEST OS_TYPE OS_VERSION"
+INSIDE_DOCKER_TRAVIS_CI_ENV="TRAVIS TRAVIS_COMMIT TRAVIS_PULL_REQUEST OS_TYPE OS_VERSION ARTIFACTNAME"
 
 COMMON_SCRIPTS="inside_docker.sh"
 
@@ -237,10 +237,8 @@ sftp_upload() {
 	if [ -n "${LATE}" ] ; then
 		sftp_cmd_pipe <<-EOF
 			cd ${DEPLOY_TO}
-
 			put ${FROM} ${TO}
 			ls -l ${TO}
-
 			symlink ${TO} ${LATE}
 			ls -l ${LATE}
 			bye
@@ -248,7 +246,6 @@ sftp_upload() {
 	else
 		sftp_cmd_pipe <<-EOF
 			cd ${DEPLOY_TO}
-
 			put ${FROM} ${TO}
 			ls -l ${TO}
 			bye
@@ -423,7 +420,7 @@ __save_env_for_docker() {
 }
 
 run_docker_script() {
-	local DOCKER_SCRIPT="$(get_script_path $1)"
+	local DOCKER_SCRIPT="$(get_script_path "$1")"
 	local MOUNTPOINT="${INSIDE_DOCKER_BUILD_DIR}"
 	local DOCKER_IMAGE="${OS_TYPE}:${OS_VERSION}"
 
