@@ -51,6 +51,18 @@ static const char * const iio_chan_type_name_spec[] = {
 	[IIO_MASSCONCENTRATION] = "massconcentration",
 };
 
+static const char * const hwmon_chan_type_name_spec[] = {
+	[HWMON_VOLTAGE] = "in",
+	[HWMON_FAN] = "fan",
+	[HWMON_PWM] = "pwm",
+	[HWMON_TEMP] = "temp",
+	[HWMON_CURRENT] = "curr",
+	[HWMON_POWER] = "power",
+	[HWMON_ENERGY] = "energy",
+	[HWMON_HUMIDITY] = "humidity",
+	[HWMON_INTRUSION] = "intrusion",
+};
+
 static const char * const modifier_names[] = {
 	[IIO_MOD_X] = "x",
 	[IIO_MOD_Y] = "y",
@@ -96,6 +108,12 @@ static const char * const modifier_names[] = {
 	[IIO_MOD_PM2P5] = "pm2p5",
 	[IIO_MOD_PM4] = "pm4",
 	[IIO_MOD_PM10] = "pm10",
+	[IIO_MOD_LINEAR_X] = "x-g",
+	[IIO_MOD_LINEAR_Y] = "y-g",
+	[IIO_MOD_LINEAR_Z] = "z-g",
+	[IIO_MOD_PITCH] = "pitch",
+	[IIO_MOD_YAW] = "yaw",
+	[IIO_MOD_ROLL] = "roll",
 };
 
 /*
@@ -124,6 +142,36 @@ unsigned int find_channel_modifier(const char *s, size_t *len_p)
 	return IIO_NO_MOD;
 }
 
+static int iio_channel_find_type(const char *id,
+			const char *const *name_spec, size_t size)
+{
+	unsigned int i;
+	size_t len;
+
+	for (i = 0; i < size; i++) {
+		len = strlen(name_spec[i]);
+		if (strncmp(name_spec[i], id, len) != 0)
+		      continue;
+
+		/* Type must be followed by one of a '\0', a '_', or a digit */
+		if (id[len] != '\0' && id[len] != '_' &&
+				(id[len] < '0' || id[len] > '9'))
+			continue;
+
+		return i;
+	}
+
+	return -EINVAL;
+}
+
+#if WITH_HWMON
+bool iio_channel_is_hwmon(const char *id)
+{
+	return iio_channel_find_type(id, hwmon_chan_type_name_spec,
+				ARRAY_SIZE(hwmon_chan_type_name_spec)) >= 0;
+}
+#endif /* WITH_HWMON */
+
 /*
  * Initializes all auto-detected fields of the channel struct. Must be called
  * after the channel has been otherwise fully initialized.
@@ -133,22 +181,19 @@ void iio_channel_init_finalize(struct iio_channel *chn)
 	unsigned int i;
 	size_t len;
 	char *mod;
+	int type;
 
-	chn->type = IIO_CHAN_TYPE_UNKNOWN;
-	chn->modifier = IIO_NO_MOD;
-
-	for (i = 0; i < ARRAY_SIZE(iio_chan_type_name_spec); i++) {
-		len = strlen(iio_chan_type_name_spec[i]);
-		if (strncmp(iio_chan_type_name_spec[i], chn->id, len) != 0)
-			continue;
-		/* Type must be followed by one of a '\0', a '_', or a digit */
-		if (chn->id[len] != '\0' && chn->id[len] != '_' &&
-				(chn->id[len] < '0' || chn->id[len] > '9'))
-			continue;
-
-		chn->type = (enum iio_chan_type) i;
+	if (iio_device_is_hwmon(chn->dev)) {
+		type = iio_channel_find_type(chn->id, hwmon_chan_type_name_spec,
+					ARRAY_SIZE(hwmon_chan_type_name_spec));
+	} else {
+		type = iio_channel_find_type(chn->id, iio_chan_type_name_spec,
+					ARRAY_SIZE(iio_chan_type_name_spec));
 	}
 
+	chn->type = (type >= 0) ? type : IIO_CHAN_TYPE_UNKNOWN;
+	chn->modifier = IIO_NO_MOD;
+
 	mod = strchr(chn->id, '_');
 	if (!mod)
 		return;
@@ -299,21 +344,55 @@ const char * iio_channel_get_attr(const struct iio_channel *chn,
 		return chn->attrs[index].name;
 }
 
-const char * iio_channel_find_attr(const struct iio_channel *chn,
-		const char *name)
+static const char *
+iio_channel_do_find_attr(const struct iio_channel *chn, const char *name)
 {
 	unsigned int i;
+
 	for (i = 0; i < chn->nb_attrs; i++) {
 		const char *attr = chn->attrs[i].name;
 		if (!strcmp(attr, name))
 			return attr;
 	}
+
+	return NULL;
+}
+
+const char * iio_channel_find_attr(const struct iio_channel *chn,
+				   const char *name)
+{
+	const char *attr;
+	size_t len;
+
+	attr = iio_channel_do_find_attr(chn, name);
+	if (attr)
+		return attr;
+
+	/* Support attribute names that start with the channel's label to avoid
+	 * breaking compatibility with old kernels, which did not offer a
+	 * 'label' attribute, and caused Libiio to sometimes misdetect the
+	 * channel's extended name as being part of the attribute name. */
+	if (chn->name) {
+		len = strlen(chn->name);
+
+		if (!strncmp(chn->name, name, len) && name[len] == '_') {
+			name += len + 1;
+			return iio_channel_do_find_attr(chn, name);
+		}
+	}
+
 	return NULL;
 }
 
 ssize_t iio_channel_attr_read(const struct iio_channel *chn,
 		const char *attr, char *dst, size_t len)
 {
+	if (attr) {
+		attr = iio_channel_find_attr(chn, attr);
+		if (!attr)
+			return -ENOENT;
+	}
+
 	if (chn->dev->ctx->ops->read_channel_attr)
 		return chn->dev->ctx->ops->read_channel_attr(chn,
 				attr, dst, len);
@@ -324,6 +403,10 @@ ssize_t iio_channel_attr_read(const struct iio_channel *chn,
 ssize_t iio_channel_attr_write_raw(const struct iio_channel *chn,
 		const char *attr, const void *src, size_t len)
 {
+	attr = iio_channel_find_attr(chn, attr);
+	if (!attr)
+		return -ENOENT;
+
 	if (chn->dev->ctx->ops->write_channel_attr)
 		return chn->dev->ctx->ops->write_channel_attr(chn,
 				attr, src, len);
