@@ -9,9 +9,11 @@
 #ifndef __IIOD_DNS_SD_H
 #define __IIOD_DNS_SD_H
 
+#include <stdint.h>
+
 struct thread_pool;
 
-void start_avahi(struct thread_pool *pool);
+void start_avahi(struct thread_pool *pool, uint16_t port);
 void stop_avahi(void);
 
 #endif /* __IIOD_DNS_SD_H */
