@@ -105,33 +105,6 @@ static int serial_get_version(const struct iio_context *ctx,
 			major, minor, git_tag);
 }
 
-static char * __serial_get_description(struct sp_port *port)
-{
-	char *description, *name, *desc;
-	size_t desc_len;
-
-	name = sp_get_port_name(port);
-	desc = sp_get_port_description(port);
-
-	desc_len = sizeof(": \0") + strlen(name) + strlen(desc);
-	description = malloc(desc_len);
-	if (!description) {
-		errno = ENOMEM;
-		return NULL;
-	}
-
-	iio_snprintf(description, desc_len, "%s: %s", name, desc);
-
-	return description;
-}
-
-static char * serial_get_description(const struct iio_context *ctx)
-{
-	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
-
-	return __serial_get_description(pdata->port);
-}
-
 static int serial_open(const struct iio_device *dev,
 		size_t samples_count, bool cyclic)
 {
@@ -355,6 +328,24 @@ static int serial_set_timeout(struct iio_context *ctx, unsigned int timeout)
 	return 0;
 }
 
+static int serial_get_trigger(const struct iio_device *dev,
+			      const struct iio_device **trigger)
+{
+	const struct iio_context *ctx = iio_device_get_context(dev);
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
+
+	return iiod_client_get_trigger(pdata->iiod_client, NULL, dev, trigger);
+}
+
+static int serial_set_trigger(const struct iio_device *dev,
+			      const struct iio_device *trigger)
+{
+	const struct iio_context *ctx = iio_device_get_context(dev);
+	struct iio_context_pdata *pdata = iio_context_get_pdata(ctx);
+
+	return iiod_client_set_trigger(pdata->iiod_client, NULL, dev, trigger);
+}
+
 static const struct iio_backend_ops serial_ops = {
 	.get_version = serial_get_version,
 	.open = serial_open,
@@ -367,8 +358,9 @@ static const struct iio_backend_ops serial_ops = {
 	.write_channel_attr = serial_write_chn_attr,
 	.set_kernel_buffers_count = serial_set_kernel_buffers_count,
 	.shutdown = serial_shutdown,
-	.get_description = serial_get_description,
 	.set_timeout = serial_set_timeout,
+	.get_trigger = serial_get_trigger,
+	.set_trigger = serial_set_trigger,
 };
 
 static const struct iiod_client_ops serial_iiod_client_ops = {
@@ -409,11 +401,10 @@ static struct iio_context * serial_create_context(const char *port_name,
 	struct sp_port *port;
 	struct iio_context_pdata *pdata;
 	struct iio_context *ctx;
-	char *description;
+	char *uri, buf[16];
 	size_t uri_len;
 	unsigned int i;
 	int ret;
-	char *uri;
 
 	uri_len = sizeof("serial:,1000000,8n1n") + strnlen(port_name, PATH_MAX);
 	uri = malloc(uri_len);
@@ -440,17 +431,25 @@ static struct iio_context * serial_create_context(const char *port_name,
 		goto err_close_port;
 	}
 
-	/* Empty the buffers */
-	sp_flush(port, SP_BUF_BOTH);
+	/* Empty the output buffer */
+	ret = libserialport_to_errno(sp_flush(port, SP_BUF_OUTPUT));
+	if (ret)
+		IIO_WARNING("Unable to flush output buffer\n");
 
-	description = __serial_get_description(port);
-	if (!description)
-		goto err_close_port;
+	/* Drain the input buffer */
+	do {
+		ret = libserialport_to_errno(sp_blocking_read(port, buf,
+							      sizeof(buf), 1));
+		if (ret < 0) {
+			IIO_WARNING("Unable to drain input buffer\n");
+			break;
+		}
+	} while (ret);
 
 	pdata = zalloc(sizeof(*pdata));
 	if (!pdata) {
 		errno = ENOMEM;
-		goto err_free_description;
+		goto err_close_port;
 	}
 
 	pdata->port = port;
@@ -467,7 +466,6 @@ static struct iio_context * serial_create_context(const char *port_name,
 	ctx->name = "serial";
 	ctx->ops = &serial_ops;
 	ctx->pdata = pdata;
-	ctx->description = description;
 
 	for (i = 0; i < iio_context_get_devices_count(ctx); i++) {
 		struct iio_device *dev = iio_context_get_device(ctx, i);
@@ -487,6 +485,15 @@ static struct iio_context * serial_create_context(const char *port_name,
 		goto err_context_destroy;
 	free(uri);
 
+	ret = iio_context_add_attr(ctx, "serial,port", sp_get_port_name(port));
+	if (ret < 0)
+		goto err_context_destroy;
+
+	ret = iio_context_add_attr(ctx, "serial,description",
+				   sp_get_port_description(port));
+	if (ret < 0)
+		goto err_context_destroy;
+
 	return ctx;
 
 err_context_destroy:
@@ -499,8 +506,6 @@ err_destroy_iiod_client:
 	iiod_client_destroy(pdata->iiod_client);
 err_free_pdata:
 	free(pdata);
-err_free_description:
-	free(description);
 err_close_port:
 	sp_close(port);
 err_free_port:
@@ -549,7 +554,7 @@ static int serial_parse_params(const char *params,
 		return -EINVAL;
 
 	/* 110 baud to 1,000,000 baud */
-	if (params == end || *baud_rate < 110 || *baud_rate > 1000001) {
+	if (params == end || *baud_rate < 110 || *baud_rate > 4000000) {
 		IIO_ERROR("Invalid baud rate\n");
 		return -EINVAL;
 	}
