@@ -1,20 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2014 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * */
+ */
 
 #define _DEFAULT_SOURCE
 
@@ -34,6 +24,10 @@
 #include <sys/sysctl.h>
 #endif
 
+#ifdef _WIN32
+#include <sysinfoapi.h>
+#endif
+
 #include "iio_common.h"
 
 #define MY_NAME "iio_stresstest"
@@ -106,15 +100,15 @@ static const struct option options[] = {
 	{"uri", required_argument, 0, 'u'},
 	{"buffer-size", required_argument, 0, 'b'},
 	{"samples", required_argument, 0, 's' },
-	{"timeout", required_argument, 0, 't'},
-	{"Threads", required_argument, 0, 'T'},
+	{"duration", required_argument, 0, 'd'},
+	{"threads", required_argument, 0, 't'},
 	{"verbose", no_argument, 0, 'v'},
 	{0, 0, 0, 0},
 };
 
 static const char *options_descriptions[] = {
-	"[-n <hostname>] [-u <vid>:<pid>] [-t <trigger>] [-b <buffer-size>] [-s <samples>]"
-		"<iio_device> [<channel> ...]",
+	("[-n <hostname>] [-u <vid>:<pid>] [-t <trigger>] [-b <buffer-size>] [-s <samples>]"
+		"<iio_device> [<channel> ...]"),
 	"Show this help and quit.",
 	"Use the context at the provided URI.",
 	"Size of the capture buffer. Default is 256.",
@@ -219,7 +213,7 @@ static void thread_err(int id, ssize_t ret, char * what)
 	if (ret < 0) {
 		char err_str[1024];
 		iio_strerror(-ret, err_str, sizeof(err_str)); \
-		fprintf(stderr, "%i : IIO ERROR : %s : %s (%zd)\n", id, what, err_str, ret); \
+		fprintf(stderr, "%i : IIO ERROR : %s : %s\n", id, what, err_str); \
 	}
 }
 
@@ -430,13 +424,13 @@ int main(int argc, char **argv)
 			info.buffer_size = sanitize_clamp("buffersize", info.argv[info.arg_index],
 					min_samples, 1024 * 1024 * 4);
 			break;
-		case 't':
+		case 'T':
 			info.arg_index +=2;
 			/* ensure between least once a day and never (0) */
 			info.timeout = 1000 * sanitize_clamp("timeout", info.argv[info.arg_index],
 					0, 60 * 60 * 24);
 			break;
-		case 'T':
+		case 't':
 			info.arg_index +=2;
 			/* Max number threads 1024, min 1 */
 			info.num_threads = sanitize_clamp("threads", info.argv[info.arg_index],
@@ -525,6 +519,12 @@ int main(int argc, char **argv)
 
 	ret = (void *)calloc(info.num_threads, sizeof(void *));
 
+	if (!ret || !info.start || !info.refills || !info.buffers || !info.starts ||
+			!info.tid || !info.threads) {
+		fprintf(stderr, "Memory allocation failure\n");
+		return 0;
+	}
+
 	for (i = 0; i < info.num_threads; i++) {
 		info.start[i] = malloc(NUM_TIMESTAMPS * sizeof(struct timeval));
 	}
@@ -664,6 +664,11 @@ int main(int argc, char **argv)
 		/* gather and sort things, so we can print out a histogram */
 		struct timeval *sort;
 		sort = calloc(info.num_threads * NUM_TIMESTAMPS, sizeof(struct timeval));
+		if (!sort) {
+			app_running = 0;
+			fprintf(stderr, "Memory allocation failure\n");
+			break;
+		}
 		b = 0;
 		/* gather */
 		for (i = 0; i < info.num_threads; i++) {
