@@ -11,6 +11,9 @@
 #include "iiod-client.h"
 
 #include <ctype.h>
+#ifdef __linux__
+#include <dirent.h>
+#endif
 #include <errno.h>
 #include <libusb.h>
 #include <stdbool.h>
@@ -928,7 +931,7 @@ static struct iio_context * usb_create_context(unsigned int bus,
 
 	pdata->nb_ep_couples = iface->bNumEndpoints / 2;
 
-	IIO_DEBUG("Found %hhu usable i/o endpoint couples\n", pdata->nb_ep_couples);
+	IIO_DEBUG("Found %hu usable i/o endpoint couples\n", pdata->nb_ep_couples);
 
 	pdata->io_endpoints = calloc(pdata->nb_ep_couples,
 			sizeof(*pdata->io_endpoints));
@@ -1200,17 +1203,75 @@ static int usb_fill_context_info(struct iio_context_info *info,
 	return 0;
 }
 
-int usb_context_scan(struct iio_scan_result *scan_result)
+static int parse_vid_pid(const char *vid_pid, uint16_t *vid, uint16_t *pid)
+{
+	unsigned long val;
+	char *ptr;
+
+	/*
+	 * vid_pid string must be either:
+	 * - NULL: scan everything,
+	 * - "vid:*": scan all devices with the given VID,
+	 * - "vid:pid": scan the device with the given VID/PID.
+	 * IDs are given in hexadecimal, and the 0x prefix is not required.
+	 */
+
+	*vid = 0;
+	*pid = 0;
+
+	if (!vid_pid)
+		return 0;
+
+	errno = 0;
+	val = strtoul(vid_pid, &ptr, 16);
+	if (ptr == vid_pid || val > 0xFFFF || *ptr != ':' || errno == ERANGE)
+		return -EINVAL;
+
+	*vid = (uint16_t) val;
+
+	vid_pid = ptr + 1;
+
+	if (*vid_pid == '*')
+		return vid_pid[1] == '\0' ? 0 : -EINVAL;
+
+	errno = 0;
+	val = strtoul(vid_pid, &ptr, 16);
+	if (ptr == vid_pid || val > 0xFFFF || *ptr != '\0' || errno == ERANGE)
+		return -EINVAL;
+
+	*pid = (uint16_t) val;
+
+	return 0;
+}
+
+int usb_context_scan(struct iio_scan_result *scan_result, const char *args)
 {
 	struct iio_context_info *info;
 	libusb_device **device_list;
 	libusb_context *ctx;
+	uint16_t vid, pid;
 	unsigned int i;
 	int ret;
 
+	ret = parse_vid_pid(args, &vid, &pid);
+	if (ret)
+		return ret;
+
 	ret = libusb_init(&ctx);
-	if (ret < 0)
+	if (ret < 0) {
+#ifdef __linux__
+		/* When Linux's OTG USB is in device mode, and there are no hosts,
+		 * libusb_init() is expected to fail, but we shouldn't treat that
+		 * as a hard failure - only that there are no devices.
+		 */
+		DIR* dir = opendir("/dev/bus/usb/");
+		if (dir)
+			closedir(dir);
+		else if (errno == ENOENT)
+			return 0;
+#endif
 		return -(int) libusb_to_errno(ret);
+	}
 
 	ret = (int) libusb_get_device_list(ctx, &device_list);
 	if (ret < 0) {
@@ -1222,6 +1283,21 @@ int usb_context_scan(struct iio_scan_result *scan_result)
 		struct libusb_device_handle *hdl;
 		struct libusb_device *dev = device_list[i];
 		unsigned int intrfc = 0;
+		struct libusb_device_descriptor device_descriptor;
+
+		/* If we are given a pid or vid, use that to qualify for things,
+		 * this avoids open/closing random devices & potentially locking
+		 * (blocking them) from other applications
+		 */
+		if(vid || pid) {
+			ret = libusb_get_device_descriptor(dev, &device_descriptor);
+			if (ret)
+				continue;
+			if (vid && vid != device_descriptor.idVendor)
+				continue;
+			if (pid && pid != device_descriptor.idProduct)
+				continue;
+		}
 
 		ret = libusb_open(dev, &hdl);
 		if (ret)
