@@ -1,20 +1,10 @@
+// SPDX-License-Identifier: LGPL-2.1-or-later
 /*
  * libiio - Library for interfacing industrial I/O (IIO) devices
  *
  * Copyright (C) 2014 Analog Devices, Inc.
  * Author: Paul Cercueil <paul.cercueil@analog.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * */
+ */
 
 #include "debug.h"
 #include "iio-private.h"
@@ -23,35 +13,31 @@
 #include <libxml/tree.h>
 #include <string.h>
 
-/* 'input' must be in UTF-8 encoded, null terminated */
-char * encode_xml_ndup(const char * input)
-{
-	char * out;
-
-	out = (char *)xmlEncodeEntitiesReentrant(NULL, (const xmlChar *)input);
-
-	return out;
-}
-
 static int add_attr_to_channel(struct iio_channel *chn, xmlNode *n)
 {
 	xmlAttr *attr;
 	char *name = NULL, *filename = NULL;
 	struct iio_channel_attr *attrs;
+	int err = -ENOMEM;
 
 	for (attr = n->properties; attr; attr = attr->next) {
 		if (!strcmp((char *) attr->name, "name")) {
 			name = iio_strdup((char *) attr->children->content);
+			if (!name)
+				goto err_free;
 		} else if (!strcmp((char *) attr->name, "filename")) {
 			filename = iio_strdup((char *) attr->children->content);
+			if (!filename)
+				goto err_free;
 		} else {
-			IIO_WARNING("Unknown field \'%s\' in channel %s\n",
-					attr->name, chn->id);
+			IIO_DEBUG("Unknown field \'%s\' in channel %s\n",
+				  attr->name, chn->id);
 		}
 	}
 
 	if (!name) {
 		IIO_ERROR("Incomplete attribute in channel %s\n", chn->id);
+		err = -EINVAL;
 		goto err_free;
 	}
 
@@ -72,78 +58,46 @@ static int add_attr_to_channel(struct iio_channel *chn, xmlNode *n)
 	return 0;
 
 err_free:
-	if (name)
-		free(name);
-	if (filename)
-		free(filename);
-	return -1;
+	free(name);
+	free(filename);
+	return err;
 }
 
 static int add_attr_to_device(struct iio_device *dev, xmlNode *n, enum iio_attr_type type)
 {
 	xmlAttr *attr;
-	char **attrs, *name = NULL;
+	char *name = NULL;
 
 	for (attr = n->properties; attr; attr = attr->next) {
 		if (!strcmp((char *) attr->name, "name")) {
-			name = iio_strdup((char *) attr->children->content);
+			name = (char *) attr->children->content;
 		} else {
-			IIO_WARNING("Unknown field \'%s\' in device %s\n",
-					attr->name, dev->id);
+			IIO_DEBUG("Unknown field \'%s\' in device %s\n",
+				  attr->name, dev->id);
 		}
 	}
 
 	if (!name) {
 		IIO_ERROR("Incomplete attribute in device %s\n", dev->id);
-		goto err_free;
+		return -EINVAL;
 	}
 
 	switch(type) {
 		case IIO_ATTR_TYPE_DEBUG:
-			attrs = realloc(dev->debug_attrs,
-					(1 + dev->nb_debug_attrs) * sizeof(char *));
-			break;
+			return add_iio_dev_attr(&dev->debug_attrs, name, " debug", dev->id);
 		case IIO_ATTR_TYPE_DEVICE:
-			attrs = realloc(dev->attrs,
-					(1 + dev->nb_attrs) * sizeof(char *));
-			break;
+			return add_iio_dev_attr(&dev->attrs, name, " ", dev->id);
 		case IIO_ATTR_TYPE_BUFFER:
-			attrs = realloc(dev->buffer_attrs,
-					(1 + dev->nb_buffer_attrs) * sizeof(char *));
-			break;
+			return add_iio_dev_attr(&dev->buffer_attrs, name, " buffer", dev->id);
 		default:
-			attrs = NULL;
-			break;
+			return -EINVAL;
 	}
-	if (!attrs)
-		goto err_free;
-
-	switch(type) {
-		case IIO_ATTR_TYPE_DEBUG:
-			attrs[dev->nb_debug_attrs++] = name;
-			dev->debug_attrs = attrs;
-			break;
-		case IIO_ATTR_TYPE_DEVICE:
-			attrs[dev->nb_attrs++] = name;
-			dev->attrs = attrs;
-			break;
-		case IIO_ATTR_TYPE_BUFFER:
-			attrs[dev->nb_buffer_attrs++] = name;
-			dev->buffer_attrs = attrs;
-			break;
-	}
-
-	return 0;
-
-err_free:
-	if (name)
-		free(name);
-	return -1;
 }
 
-static void setup_scan_element(struct iio_channel *chn, xmlNode *n)
+static int setup_scan_element(struct iio_channel *chn, xmlNode *n)
 {
 	xmlAttr *attr;
+	int err;
 
 	for (attr = n->properties; attr; attr = attr->next) {
 		const char *name = (const char *) attr->name,
@@ -155,12 +109,12 @@ static void setup_scan_element(struct iio_channel *chn, xmlNode *n)
 			errno = 0;
 			value = strtoll(content, &end, 0);
 			if (end == content || value < 0 || errno == ERANGE)
-				return;
+				return -EINVAL;
 			chn->index = (long) value;
 		} else if (!strcmp(name, "format")) {
 			char e, s;
 			if (strchr(content, 'X')) {
-				iio_sscanf(content, "%ce:%c%u/%uX%u>>%u",
+				err = iio_sscanf(content, "%ce:%c%u/%uX%u>>%u",
 #ifdef _MSC_BUILD
 					&e, (unsigned int)sizeof(e),
 					&s, (unsigned int)sizeof(s),
@@ -171,9 +125,11 @@ static void setup_scan_element(struct iio_channel *chn, xmlNode *n)
 					&chn->format.length,
 					&chn->format.repeat,
 					&chn->format.shift);
+				if (err != 6)
+					return -EINVAL;
 			} else {
 				chn->format.repeat = 1;
-				iio_sscanf(content, "%ce:%c%u/%u>>%u",
+				err = iio_sscanf(content, "%ce:%c%u/%u>>%u",
 #ifdef _MSC_BUILD
 					&e, (unsigned int)sizeof(e),
 					&s, (unsigned int)sizeof(s),
@@ -183,6 +139,8 @@ static void setup_scan_element(struct iio_channel *chn, xmlNode *n)
 					&chn->format.bits,
 					&chn->format.length,
 					&chn->format.shift);
+				if (err != 5)
+					return -EINVAL;
 			}
 			chn->format.is_be = e == 'b';
 			chn->format.is_signed = (s == 's' || s == 'S');
@@ -196,24 +154,29 @@ static void setup_scan_element(struct iio_channel *chn, xmlNode *n)
 			value = strtof(content, &end);
 			if (end == content || errno == ERANGE) {
 				chn->format.with_scale = false;
-				return;
+				return -EINVAL;
 			}
 
 			chn->format.with_scale = true;
 			chn->format.scale = value;
 		} else {
-			IIO_WARNING("Unknown attribute \'%s\' in <scan-element>\n",
-					name);
+			IIO_DEBUG("Unknown attribute \'%s\' in <scan-element>\n",
+				  name);
 		}
 	}
+
+	return 0;
 }
 
 static struct iio_channel * create_channel(struct iio_device *dev, xmlNode *n)
 {
 	xmlAttr *attr;
-	struct iio_channel *chn = zalloc(sizeof(*chn));
+	struct iio_channel *chn;
+	int err = -ENOMEM;
+
+	chn = zalloc(sizeof(*chn));
 	if (!chn)
-		return NULL;
+		return ERR_PTR(-ENOMEM);
 
 	chn->dev = dev;
 
@@ -225,34 +188,42 @@ static struct iio_channel * create_channel(struct iio_device *dev, xmlNode *n)
 		      *content = (const char *) attr->children->content;
 		if (!strcmp(name, "name")) {
 			chn->name = iio_strdup(content);
+			if (!chn->name)
+				goto err_free_channel;
 		} else if (!strcmp(name, "id")) {
 			chn->id = iio_strdup(content);
+			if (!chn->id)
+				goto err_free_channel;
 		} else if (!strcmp(name, "type")) {
 			if (!strcmp(content, "output"))
 				chn->is_output = true;
 			else if (strcmp(content, "input"))
-				IIO_WARNING("Unknown channel type %s\n", content);
+				IIO_DEBUG("Unknown channel type %s\n", content);
 		} else {
-			IIO_WARNING("Unknown attribute \'%s\' in <channel>\n",
-					name);
+			IIO_DEBUG("Unknown attribute \'%s\' in <channel>\n",
+				  name);
 		}
 	}
 
 	if (!chn->id) {
 		IIO_ERROR("Incomplete <attribute>\n");
+		err = -EINVAL;
 		goto err_free_channel;
 	}
 
 	for (n = n->children; n; n = n->next) {
 		if (!strcmp((char *) n->name, "attribute")) {
-			if (add_attr_to_channel(chn, n) < 0)
+			err = add_attr_to_channel(chn, n);
+			if (err < 0)
 				goto err_free_channel;
 		} else if (!strcmp((char *) n->name, "scan-element")) {
 			chn->is_scan_element = true;
-			setup_scan_element(chn, n);
+			err = setup_scan_element(chn, n);
+			if (err < 0)
+				goto err_free_channel;
 		} else if (strcmp((char *) n->name, "text")) {
-			IIO_WARNING("Unknown children \'%s\' in <channel>\n",
-					n->name);
+			IIO_DEBUG("Unknown children \'%s\' in <channel>\n",
+				  n->name);
 			continue;
 		}
 	}
@@ -263,15 +234,18 @@ static struct iio_channel * create_channel(struct iio_device *dev, xmlNode *n)
 
 err_free_channel:
 	free_channel(chn);
-	return NULL;
+	return ERR_PTR(err);
 }
 
 static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 {
 	xmlAttr *attr;
-	struct iio_device *dev = zalloc(sizeof(*dev));
+	struct iio_device *dev;
+	int err = -ENOMEM;
+
+	dev = zalloc(sizeof(*dev));
 	if (!dev)
-		return NULL;
+		return ERR_PTR(-ENOMEM);
 
 	dev->ctx = ctx;
 
@@ -279,16 +253,25 @@ static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 		if (!strcmp((char *) attr->name, "name")) {
 			dev->name = iio_strdup(
 					(char *) attr->children->content);
+			if (!dev->name)
+				goto err_free_device;
+		} else if (!strcmp((char *) attr->name, "label")) {
+			dev->label = iio_strdup((char *) attr->children->content);
+			if (!dev->label)
+				goto err_free_device;
 		} else if (!strcmp((char *) attr->name, "id")) {
 			dev->id = iio_strdup((char *) attr->children->content);
+			if (!dev->id)
+				goto err_free_device;
 		} else {
-			IIO_WARNING("Unknown attribute \'%s\' in <device>\n",
-					attr->name);
+			IIO_DEBUG("Unknown attribute \'%s\' in <device>\n",
+				  attr->name);
 		}
 	}
 
 	if (!dev->id) {
 		IIO_ERROR("Unable to read device ID\n");
+		err = -EINVAL;
 		goto err_free_device;
 	}
 
@@ -296,14 +279,16 @@ static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 		if (!strcmp((char *) n->name, "channel")) {
 			struct iio_channel **chns,
 					   *chn = create_channel(dev, n);
-			if (!chn) {
-				IIO_ERROR("Unable to create channel\n");
+			if (IS_ERR(chn)) {
+				err = PTR_ERR(chn);
+				IIO_ERROR("Unable to create channel: %d\n", err);
 				goto err_free_device;
 			}
 
 			chns = realloc(dev->channels, (1 + dev->nb_channels) *
 					sizeof(struct iio_channel *));
 			if (!chns) {
+				err = -ENOMEM;
 				IIO_ERROR("Unable to allocate memory\n");
 				free(chn);
 				goto err_free_device;
@@ -312,17 +297,20 @@ static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 			chns[dev->nb_channels++] = chn;
 			dev->channels = chns;
 		} else if (!strcmp((char *) n->name, "attribute")) {
-			if (add_attr_to_device(dev, n, IIO_ATTR_TYPE_DEVICE) < 0)
+			err = add_attr_to_device(dev, n, IIO_ATTR_TYPE_DEVICE);
+			if (err < 0)
 				goto err_free_device;
 		} else if (!strcmp((char *) n->name, "debug-attribute")) {
-			if (add_attr_to_device(dev, n, IIO_ATTR_TYPE_DEBUG) < 0)
+			err = add_attr_to_device(dev, n, IIO_ATTR_TYPE_DEBUG);
+			if (err < 0)
 				goto err_free_device;
 		} else if (!strcmp((char *) n->name, "buffer-attribute")) {
-			if (add_attr_to_device(dev, n, IIO_ATTR_TYPE_BUFFER) < 0)
+			err = add_attr_to_device(dev, n, IIO_ATTR_TYPE_BUFFER);
+			if (err < 0)
 				goto err_free_device;
 		} else if (strcmp((char *) n->name, "text")) {
-			IIO_WARNING("Unknown children \'%s\' in <device>\n",
-					n->name);
+			IIO_DEBUG("Unknown children \'%s\' in <device>\n",
+				  n->name);
 			continue;
 		}
 	}
@@ -331,7 +319,7 @@ static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 	if (dev->words) {
 		dev->mask = calloc(dev->words, sizeof(*dev->mask));
 		if (!dev->mask) {
-			errno = ENOMEM;
+			err = -ENOMEM;
 			goto err_free_device;
 		}
 	}
@@ -340,7 +328,8 @@ static struct iio_device * create_device(struct iio_context *ctx, xmlNode *n)
 
 err_free_device:
 	free_device(dev);
-	return NULL;
+
+	return ERR_PTR(err);
 }
 
 static struct iio_context * xml_clone(const struct iio_context *ctx)
@@ -352,6 +341,13 @@ static const struct iio_backend_ops xml_ops = {
 	.clone = xml_clone,
 };
 
+static const struct iio_backend xml_backend = {
+	.api_version = IIO_BACKEND_API_V1,
+	.name = "xml",
+	.uri_prefix = "xml:",
+	.ops = &xml_ops,
+};
+
 static int parse_context_attr(struct iio_context *ctx, xmlNode *n)
 {
 	xmlAttr *attr;
@@ -371,91 +367,108 @@ static int parse_context_attr(struct iio_context *ctx, xmlNode *n)
 		return iio_context_add_attr(ctx, name, value);
 }
 
-static struct iio_context * iio_create_xml_context_helper(xmlDoc *doc)
+static int iio_populate_xml_context_helper(struct iio_context *ctx, xmlNode *root)
 {
-	unsigned int i;
-	xmlNode *root, *n;
-	xmlAttr *attr;
-	int err = -ENOMEM;
-	struct iio_context *ctx = zalloc(sizeof(*ctx));
-	if (!ctx)
-		goto err_set_errno;
-
-	ctx->name = "xml";
-	ctx->ops = &xml_ops;
-
-	root = xmlDocGetRootElement(doc);
-	if (strcmp((char *) root->name, "context")) {
-		IIO_ERROR("Unrecognized XML file\n");
-		err = -EINVAL;
-		goto err_free_ctx;
-	}
-
-	for (attr = root->properties; attr; attr = attr->next) {
-		if (!strcmp((char *) attr->name, "description"))
-			ctx->description = iio_strdup(
-					(char *) attr->children->content);
-		else if (strcmp((char *) attr->name, "name"))
-			IIO_WARNING("Unknown parameter \'%s\' in <context>\n",
-					(char *) attr->children->content);
-	}
+	xmlNode *n;
+	int err;
 
 	for (n = root->children; n; n = n->next) {
-		struct iio_device **devs, *dev;
+		struct iio_device *dev;
 
 		if (!strcmp((char *) n->name, "context-attribute")) {
 			err = parse_context_attr(ctx, n);
 			if (err)
-				goto err_free_devices;
-			else
-				continue;
+				return err;
+
+			continue;
 		} else if (strcmp((char *) n->name, "device")) {
 			if (strcmp((char *) n->name, "text"))
-				IIO_WARNING("Unknown children \'%s\' in "
-						"<context>\n", n->name);
+				IIO_DEBUG("Unknown children \'%s\' in "
+					  "<context>\n", n->name);
 			continue;
 		}
 
 		dev = create_device(ctx, n);
-		if (!dev) {
-			IIO_ERROR("Unable to create device\n");
-			goto err_free_devices;
+		if (IS_ERR(dev)) {
+			err = PTR_ERR(dev);
+			IIO_ERROR("Unable to create device: %d\n", err);
+			return err;
 		}
 
-		devs = realloc(ctx->devices, (1 + ctx->nb_devices) *
-				sizeof(struct iio_device *));
-		if (!devs) {
-			IIO_ERROR("Unable to allocate memory\n");
+		err = iio_context_add_device(ctx, dev);
+		if (err) {
 			free(dev);
-			goto err_free_devices;
+			return err;
 		}
-
-		devs[ctx->nb_devices++] = dev;
-		ctx->devices = devs;
 	}
 
-	err = iio_context_init(ctx);
-	if (err)
-		goto err_free_devices;
+	return iio_context_init(ctx);
+}
+
+static struct iio_context * iio_create_xml_context_helper(xmlDoc *doc)
+{
+	const char *description = NULL, *git_tag = NULL, *content;
+	struct iio_context *ctx;
+	long major = 0, minor = 0;
+	xmlNode *root;
+	xmlAttr *attr;
+	char *end;
+	int err;
+
+	root = xmlDocGetRootElement(doc);
+	if (strcmp((char *) root->name, "context")) {
+		IIO_ERROR("Unrecognized XML file\n");
+		errno = EINVAL;
+		return NULL;
+	}
+
+	for (attr = root->properties; attr; attr = attr->next) {
+		content = (const char *) attr->children->content;
+
+		if (!strcmp((char *) attr->name, "description")) {
+			description = content;
+		} else if (!strcmp((char *) attr->name, "version-major")) {
+			errno = 0;
+			major = strtol(content, &end, 10);
+			if (*end != '\0' ||  errno == ERANGE)
+				IIO_WARNING("invalid format for major version\n");
+		} else if (!strcmp((char *) attr->name, "version-minor")) {
+			errno = 0;
+			minor = strtol(content, &end, 10);
+			if (*end != '\0' || errno == ERANGE)
+				IIO_WARNING("invalid format for minor version\n");
+		} else if (!strcmp((char *) attr->name, "version-git")) {
+			git_tag = content;
+		} else if (strcmp((char *) attr->name, "name")) {
+			IIO_DEBUG("Unknown parameter \'%s\' in <context>\n",
+				  content);
+		}
+	}
+
+	ctx = iio_context_create_from_backend(&xml_backend, description);
+	if (!ctx)
+		return NULL;
+
+	if (git_tag) {
+		ctx->major = major;
+		ctx->minor = minor;
+
+		ctx->git_tag = iio_strdup(git_tag);
+		if (!ctx->git_tag) {
+			iio_context_destroy(ctx);
+			errno = ENOMEM;
+			return NULL;
+		}
+	}
+
+	err = iio_populate_xml_context_helper(ctx, root);
+	if (err) {
+		iio_context_destroy(ctx);
+		errno = -err;
+		return NULL;
+	}
 
 	return ctx;
-
-err_free_devices:
-	for (i = 0; i < ctx->nb_devices; i++)
-		free_device(ctx->devices[i]);
-	if (ctx->nb_devices)
-		free(ctx->devices);
-	for (i = 0; i < ctx->nb_attrs; i++) {
-		free(ctx->attrs[i]);
-		free(ctx->values[i]);
-	}
-	free(ctx->attrs);
-	free(ctx->values);
-err_free_ctx:
-	free(ctx);
-err_set_errno:
-	errno = -err;
-	return NULL;
 }
 
 struct iio_context * xml_create_context(const char *xml_file)
@@ -495,3 +508,47 @@ struct iio_context * xml_create_context_mem(const char *xml, size_t len)
 	xmlFreeDoc(doc);
 	return ctx;
 }
+
+
+static void cleanup_libxml2_stuff(void)
+{
+	/*
+	 * This function will be called only when the libiio library is
+	 * unloaded (e.g. when the program exits).
+	 *
+	 * Cleanup libxml2 so that memory analyzer tools like Valgrind won't
+	 * detect a memory leak.
+	 */
+	xmlCleanupParser();
+	xmlMemoryDump();
+}
+
+#if defined(_MSC_BUILD)
+#pragma section(".CRT$XCU", read)
+#define __CONSTRUCTOR(f, p) \
+  static void f(void); \
+  __declspec(allocate(".CRT$XCU")) void (*f##_)(void) = f; \
+  __pragma(comment(linker,"/include:" p #f "_")) \
+  static void f(void)
+#ifdef _WIN64
+#define _CONSTRUCTOR(f) __CONSTRUCTOR(f, "")
+#else
+#define _CONSTRUCTOR(f) __CONSTRUCTOR(f, "_")
+#endif
+#elif defined(__GNUC__)
+#define _CONSTRUCTOR(f) static void __attribute__((constructor)) f(void)
+#else
+#define _CONSTRUCTOR(f) static void f(void)
+#endif
+
+_CONSTRUCTOR(initialize)
+{
+	/*
+	 * When the library loads, register our destructor.
+	 * Do it here and not in the context creation function,
+	 * as it could otherwise end up registering the destructor
+	 * many times.
+	 */
+	atexit(cleanup_libxml2_stuff);
+}
+#undef _CONSTRUCTOR
